Imports System.Drawing.Drawing2D

Module VBSFCRegionData
    'This module is an example of how to read the .dat Data files
    'saved by VBSFC version 6 to produce a region.

    'The .dat binary file format is as follows:
    'The leftmost 5 bytes must be "VBSFC"
    '
    'The next 2 bytes are the Major and Minor versions of VBSFC that wrote the file.
    'The 8th and 9th bytes form an integer value specifying the largest number of points
    'in any polygon.  This is handy if you want to define an array to hold polygon points
    'and don't want to have to resize it.
    '
    'Subsequent bytes (10 and onwards) follow the fixed format:
    'ObjectType,Top,Left,Right,Bottom,RoundedRectX,RoundedRectY,CombinationMethod
    'Where ObjectType and CombinationMethod are single bytes and all the other are double
    'bytes (integers).  ObjectType and CombinationMethod values are defined in the Enum
    'structures below for ObjectTypes and Methods.  If any property is not used it
    'is 0, and still takes up the same 2 bytes, so each record is of the same length.
    '
    'In the case of a polygon object type the fixed length object bytes are followed by
    'an integer which specifies the length (in integers, so double this number of bytes)
    'of a variable length section containing integer points in the polygon, X followed by Y.
    'For polygons the Top property specifies Winding or Alternate fill.
    '
    'To apply the resulting region to a form, put the line:
    '   Region = ReadFormRegion(strFilename)
    'in the Form Load event.

    Private Enum ObjectTypes
        Rectangle = 0
        Polygon = 1
        RoundedRectangle = 2
        Ellipse = 3
    End Enum
    Private Enum Methods
        ANDCombine = 0
        ORCombine = 1
        XORCombine = 2
        NOTCombine = 3
    End Enum

    Public Function ReadFormRegion(ByVal strFilename As String) As Region

        Dim strBuffer As String, byteBuffer As Byte, intBuffer As Int16
        Dim ObjectType As ObjectTypes, Top As Int16, Left As Int16, Right As Int16, Bottom As Int16, RoundedRectX As Int16, RoundedRectY As Int16, Method As Methods
        Dim Counter As Int16, PolyCount As Int16
        Dim bFirst As Boolean
        Dim rgnResult As Region
        Dim gPath As GraphicsPath
        Dim PolyPoints() As PointF

        rgnResult = New Region()
        gPath = New GraphicsPath()

        bFirst = True

        FileOpen(1, strFilename, OpenMode.Binary)

        'Open the file

        strBuffer = "*****"    'Read 5 bytes
        FileGet(1, strBuffer)

        'Check for validity
        If strBuffer <> "VBSFC" Then
            MsgBox(strFilename + " is not a valid data file.", vbOKOnly Or vbCritical, "VB Shaped Form Creator")
            FileClose(1)
            Exit Function
        End If

        'Create Polygon Points buffer
        Seek(1, 8)

        FileGet(1, intBuffer)

        Do
            FileGet(1, ByteBuffer) : ObjectType = ByteBuffer
            FileGet(1, intBuffer) : Top = intBuffer
            FileGet(1, intBuffer) : Left = intBuffer
            FileGet(1, intBuffer) : Right = intBuffer
            FileGet(1, intBuffer) : Bottom = intBuffer
            FileGet(1, intBuffer) : RoundedRectX = intBuffer
            FileGet(1, intBuffer) : RoundedRectY = intBuffer
            FileGet(1, ByteBuffer) : Method = ByteBuffer

            Select Case ObjectType

                Case ObjectType.Rectangle

                    gPath = New GraphicsPath()

                    gPath.AddRectangle(New Rectangle(Left, Top, Right - Left, Bottom - Top))

                    If bFirst Then
                        bFirst = False
                        rgnResult = New Region(gPath)
                    Else
                        AddPathToRegion(rgnResult, gPath, Method)
                    End If

                    gPath.Dispose()

                Case ObjectType.Polygon

                    gPath = New GraphicsPath()

                    FileGet(1, PolyCount)

                    ReDim PolyPoints(PolyCount - 1)

                    PolyPoints.Clear(PolyPoints, 0, PolyPoints.Length)

                    If PolyCount > 0 Then
                        For Counter = 0 To PolyCount - 1
                            FileGet(1, intBuffer) : PolyPoints(Counter).X = intBuffer
                            FileGet(1, intBuffer) : PolyPoints(Counter).Y = intBuffer
                        Next Counter

                        Select Case Top
                            Case 1
                                gPath.FillMode = FillMode.Alternate
                            Case 2
                                gPath.FillMode = FillMode.Winding
                        End Select

                        gPath.AddPolygon(PolyPoints)
                        gPath.CloseFigure()

                        If bFirst Then
                            bFirst = False
                            rgnResult = New Region(gPath)
                        Else
                            AddPathToRegion(rgnResult, gPath, Method)
                        End If

                    End If

                    gPath.Dispose()

                Case ObjectType.RoundedRectangle

                    gPath = New GraphicsPath()
                    gPath = RoundedRect(Left, Top, (Right - Left), Bottom - Top, RoundedRectX, RoundedRectY)

                    If bFirst Then
                        bFirst = False
                        rgnResult = New Region(gPath)
                    Else
                        AddPathToRegion(rgnResult, gPath, Method)
                    End If

                    gPath.Dispose()

                Case ObjectType.Ellipse

                    gPath = New GraphicsPath()
                    gPath.AddEllipse(New Rectangle(Left, Top, (Right - Left) - 1, (Bottom - Top) - 2))

                    If bFirst Then
                        bFirst = False
                        rgnResult = New Region(gPath)
                    Else
                        AddPathToRegion(rgnResult, gPath, Method)
                    End If

                    gPath.Dispose()

            End Select

        Loop Until Loc(1) = LOF(1)

        FileClose(1)

        ReadFormRegion = rgnResult

    End Function

    Private Sub AddPathToRegion(ByRef rgnResult, ByVal gPath, ByVal Method)
        Select Case Method
            Case Methods.ANDCombine
                rgnResult.intersect(gPath)
            Case Methods.NOTCombine
                rgnResult.Exclude(gPath)
            Case Methods.ORCombine
                rgnResult.Union(gPath)
            Case Methods.XORCombine
                rgnResult.Xor(gPath)
        End Select
    End Sub

    Private Function RoundedRect(ByVal x As Integer, ByVal y As Integer, ByVal width As Integer, ByVal height As Integer, ByVal xrad As Integer, ByVal yrad As Integer) As GraphicsPath

        Dim gPath As GraphicsPath
        Dim cornerwidth As Integer, cornerheight As Integer

        width = width - 1
        height = height + 1

        cornerwidth = xrad * 2
        cornerheight = yrad * 2

        gPath = New GraphicsPath()
        gPath.AddArc(x, y, cornerwidth, cornerheight, 180, 90)
        gPath.AddArc(x + width - cornerwidth, y, cornerwidth, cornerheight, 270, 90)
        gPath.AddArc(x + width - cornerwidth, y + height - cornerheight, cornerwidth, cornerheight, 0, 90)
        gPath.AddArc(x, y + height - cornerheight, cornerwidth, cornerheight, 90, 90)
        gPath.CloseFigure()
        Return gPath

    End Function



End Module

